import { NostrChannel } from './channel.js';
import { EventEmitter } from './emitter.js';
import { NostrSocket } from './socket.js';
import { NostrStore } from './store.js';
const ROOM_DEFAULTS = () => {
    return {
        debug: false,
        echo: false,
        verbose: false
    };
};
export class NostrRoom extends EventEmitter {
    static { this.list = NostrStore.list; }
    constructor(signer, options) {
        const opt = { ...ROOM_DEFAULTS(), ...options };
        const socket = new NostrSocket(opt);
        super();
        this._opt = opt;
        this._socket = socket;
        this._store = new NostrStore(signer, { ...opt, socket });
        this._sub = new NostrChannel(signer, { ...opt, socket });
        this._init = false;
        this._store.once('ready', () => void this._ready_check());
        this._store.on('fetch', () => void this.emit('fetch', this));
        this._store.on('update', () => void this.emit('update', this));
        this._sub.once('ready', () => void this._ready_check());
        this._sub.on('msg', (msg) => void this.emit('msg', msg));
    }
    get data() {
        return this._store.data;
    }
    get socket() {
        if (this._socket === null) {
            throw new Error('socket not initialized');
        }
        return this._socket;
    }
    get store() {
        return this._store.store;
    }
    _initialize(secret) {
        this._store._initialize(secret);
        this._sub._initialize(secret);
        this.socket.on('close', () => void this.emit('close', this));
    }
    _ready_check() {
        if (this._store.ready && this._sub.ready) {
            this._init = true;
            this.emit('ready', this);
        }
    }
    close() {
        this.socket.close();
        return this;
    }
    async connect(address, secret) {
        this._initialize(secret);
        this._store._fetch();
        this._socket.connect(address);
        return this;
    }
    delete() {
        return this._store.delete();
    }
    fetch() {
        return this._store.fetch();
    }
    async init(address, data, secret) {
        this._initialize(secret);
        this._store.update(data);
        this._socket.connect(address);
        return this;
    }
    on_topic(topic, fn) {
        return this._sub.on_topic(topic, fn);
    }
    refresh() {
        return this._store.refresh();
    }
    send(subject, body) {
        return this._sub.send(subject, body);
    }
    update(data, tags, updated_at) {
        return this._store.update(data, tags, updated_at);
    }
}
